#once

#include "library.s"
#include "lcd5110.s"

; Library Functions
; Compute_Pi_32
; Compute_Pi_64

;;;;;;;;;;;;;;;;;;;;;;;;

Compute_Pi_Str1:
    #d "Computing...", 0x00
Compute_Pi_Str2:
    #d "pi = 3.", 0x00
Compute_Pi_Str3:
    #d "iter.= ", 0x00

Compute_Pi_Init:
    PUSH_PC
    CALL LCD_Clear
    LCD_SetPos 0, 0
    LCD_WriteStrF Compute_Pi_Str1
    LCD_SetPos 0, 2
    LCD_WriteStrF Compute_Pi_Str2
    LCD_SetPos 0, 5
    LCD_WriteStrF Compute_Pi_Str3
    POP_PC
    RET

Compute_Pi_32:
    PUSH_PC
    PUSH RF     ; 32/64 selection
    LDI RF, 0
Compute_Pi_64_entry:
    PUSHW RA    ; Divisor
    PUSHW RB    ; Accumulator 32
    PUSHW WA    ; Accumulator 64
    PUSHW WA+4  ; Accumulator 64
    PUSH RC    ; Iterations
    CALL Compute_Pi_Init
    LDIW RA, 2
    LDIW RB, 0
    LDIW WA, 0
    LDIW WA+4, 0
    LDI RC, 0
Compute_Pi_32_loop:
    MOVW FA, RA
    INC RA
    MOV FB, RA
    CALL Multiply32_32x16
    INC RA
    BRNE RF, Compute_Pi_64_Iter
    MOV FB, RA
    CALL Multiply32_32x16
    MOVW FB, FA
    LDIW FA, 4000000000
    CALL Divide32_32x32
    ANDI TMP, RC, 1
    BRFS .add
    SUB32 RB, FA
    JMP .continue
.add:
    ADD32 RB, FA
.continue:
    MOVW FA, RB
Compute_Pi_64_continue:
    INC RC
    MOV FB, RC
    CALL Compute_Pi_Print
    ;;;
    JMP Compute_Pi_32_loop
    ;;;
    LDI TMP, 0 ; For some reason, it fixes button press
    ANDI TMP, 0, 2
    BRFS Compute_Pi_32_loop
Compute_Pi_Stop:
    POP RC
    POPW WA+4
    POPW WA
    POPW RB
    POPW RA
    POP RF
    POP_PC
    RET

Compute_Pi_64:
    PUSH_PC
    PUSH RF     ; 32/64 selection
    LDI RF, 1
    JMP Compute_Pi_64_entry
Compute_Pi_64_Iter:
    MOVW WFA, FA
    LDIW WFA+4, 0
    MOV FA, RA
    CALL Multiply64_64x16
    MOVW WFB, WFA
    MOVW WFB+4, WFA+4
    LDIW WFA,   (4000000000000000000)&0xFFFFFFFF
    LDIW WFA+4, (4000000000000000000)>>32
    CALL Divide64_64x64
    ANDI TMP, RC, 1
    BRFS .add
    SUB32 WA, WFA
    SBC WA+4, WFA+4
    SBC WA+6, WFA+6
    JMP .continue
.add:
    ADD32 WA, WFA
    ADC WA+4, WFA+4
    ADC WA+6, WFA+6
.continue:
    MOVW WFA, WA
    MOVW WFA+4, WA+4
    JMP Compute_Pi_64_continue

; Print FA32/WFA64 = pi, FB16 = iter
Compute_Pi_Print:
    PUSH_PC
    PUSHW RA
    PUSH RB
    PUSH RC
    MOVW RA, FA
    MOV RB, FB
    ; Print iterations
    LDI RC, 3-1
.loop1:
    ADDI FA, RC, 7
    CALL LCD_SetPos
    LDI FA, 5
    CALL LCD_SetY
    MOV FA, RB
    LDI FB, 10
    CALL Divide16
    MOV RB, FA
    LSR FA, FC
    ADDI FA, 48
    CALL LCD_WriteChar
    DEC RC
    BRFC .loop1
    ; Print pi
    BRNE RF, .print64
    MOVW FA, RA
    LDI FB, 100
    CALL Divide32_32x16
    MOVW RA, FA
    LDI RC, 7-1
.loop2:
    LDI FA, 2
    CALL LCD_SetY
    ADDI FA, RC, 7
    CALL LCD_SetPos
    MOVW FA, RA
    LDI FB, 10
    CALL Divide32_32x16
    MOVW RA, FA
    LSR FA, FC
    ADDI FA, 48
    CALL LCD_WriteChar
    DEC RC
    BRFC .loop2
    JMP .stop
.print64:
    LDIW WFB, (10000)&0xFFFFFFFF
    LDIW WFB+4, 0
    CALL Divide64_64x64
    LDIW WFB, (10)&0xFFFFFFFF
    LDI RC, 14-1
.print64_loop:
    LDI FA, 3
    LDI FB, 0
    SUBI FC, RC, 7
    BRFC .set_pos
    LDI FA, 2
    LDI FB, 7
.set_pos:
    CALL LCD_SetY
    ADD FA, FB, RC
    CALL LCD_SetPos
    CALL Divide64_64x64
    LSR FA, WFC
    ADDI FA, 48
    CALL LCD_WriteChar
    DEC RC
    BRFC .print64_loop
.stop:
    POP RC
    POP RB
    POPW RA
    POP_PC
    RET
