#once

#include "library.s"
#include "lcd5110.s"

; Library Functions
; Cube3D

;;;;;;;;;;;;;;;;;;;;;;;;

GFX_FOV    = 6
GFX_SCALE  = 12

; Wireframe
GFX_WIREFRAME_NODES_LEN = 8
GFX_WIREFRAME_NODES:
    #d16     FP_ONE,  FP_ONE, FP_ONE,  FP_ONE,  FP_ONE, -FP_ONE
    #d16     FP_ONE, -FP_ONE, FP_ONE,  FP_ONE, -FP_ONE, -FP_ONE
    #d16    -FP_ONE, -FP_ONE, FP_ONE, -FP_ONE, -FP_ONE, -FP_ONE
    #d16    -FP_ONE,  FP_ONE, FP_ONE, -FP_ONE,  FP_ONE, -FP_ONE
GFX_WIREFRAME_LINES_LEN = 14
GFX_WIREFRAME_LINES:
    #d8     0, 2, 2, 4, 4, 6, 6, 0
    #d8     1, 3, 3, 5, 5, 7, 7, 1
    #d8     0, 1, 2, 3, 4, 5, 6, 7
    #d8     1, 5, 3, 7

; Static Memory pointers
GFX_PTR1    = RAM_ADDR_USER + 0         ; ptr
GFX_PTR2    = RAM_ADDR_USER + 2         ; ptr
GFX_PTR3    = RAM_ADDR_USER + 4         ; ptr
GFX_PTR4    = RAM_ADDR_USER + 6         ; ptr
GFX_ROTATEX = RAM_ADDR_USER + 8         ; Rotate
GFX_ROTATEY = RAM_ADDR_USER + 10        ; Rotate
GFX_MATRIXW = RAM_ADDR_USER + 12        ; FP16 * 4 * 4 = 32
GFX_MATRIXA = GFX_MATRIXW + 32          ; FP16 * 4 * 4 = 32
GFX_NODES   = GFX_MATRIXA + 32          ; GFX_WIREFRAME_LEN (2*points)

Cube3D:
    PUSH_PC
    LDI GFX_ROTATEX, 30
    LDI GFX_ROTATEY, 0
.loop:
    CALL Cube3D_Display
    ADDI GFX_ROTATEY, 5
    BRLOI GFX_ROTATEY, 360, .angle_ok
    SUBI GFX_ROTATEY, 360
.angle_ok:
    JMP .loop
    POP_PC
    RET

Cube3D_Display:
    PUSH_PC
    ; Rx registers are not saved

    CALL GFX_Clear

    ; World Matrix
    LDI FA, GFX_MATRIXW
    LDI FB, 32
    CALL MemClr
    LDI GFX_MATRIXW + 8*1 + 2*1, FP_ONE
    LDI GFX_MATRIXW + 8*3 + 2*3, FP_ONE
    MOV FA, GFX_ROTATEY
    CALL CosFP
    MOV GFX_MATRIXW + 8*0 + 2*0, FA
    MOV GFX_MATRIXW + 8*2 + 2*2, FA
    MOV FA, GFX_ROTATEY
    CALL SinFP
    MOV GFX_MATRIXW + 8*2 + 2*0, FA
    NEG FA
    MOV GFX_MATRIXW + 8*0 + 2*2, FA
    ; Rotation Y
    LDI FA, GFX_MATRIXA
    LDI FB, 32
    CALL MemClr
    LDI GFX_MATRIXA + 8*0 + 2*0, FP_ONE
    LDI GFX_MATRIXA + 8*3 + 2*3, FP_ONE
    MOV FA, GFX_ROTATEX
    CALL CosFP
    MOV GFX_MATRIXA + 8*1 + 2*1, FA
    MOV GFX_MATRIXA + 8*2 + 2*2, FA
    MOV FA, GFX_ROTATEX
    CALL SinFP
    MOV GFX_MATRIXA + 8*1 + 2*2, FA
    NEG FA
    MOV GFX_MATRIXA + 8*2 + 2*1, FA
    ; Multiplication
    LDI GFX_PTR1, GFX_MATRIXW
.rotate_loop1:
    LDI GFX_PTR2, GFX_MATRIXA
    LDI RB, GFX_MATRIXA + 8*3
.rotate_loop2:
    MOV GFX_PTR3, GFX_PTR1
    MOV GFX_PTR4, GFX_PTR2
    LDIW RA, 0
    LDI RC, 3-1
.rotate_loop3:
    LD FA, GFX_PTR3
    LD FB, GFX_PTR4
    CALL Multiply32_16x16_signed
    ADD RA, FA
    ADC RA+2, FA+2
    ADDI GFX_PTR3, 2
    ADDI GFX_PTR4, 8
    DEC RC
    BRFC .rotate_loop3
    LSL RA, RA+2
    LSL RA
    ST RB, RA
    ADDI RB, 2
    ADDI GFX_PTR2, 2
    SUBI TMP, GFX_PTR2, GFX_MATRIXA+2*3
    BRFS .rotate_loop2
    ST GFX_PTR1, GFX_MATRIXA + 8*3 + 2*0
    ADDI GFX_PTR1, 2
    ST GFX_PTR1, GFX_MATRIXA + 8*3 + 2*1
    ADDI GFX_PTR1, 2
    ST GFX_PTR1, GFX_MATRIXA + 8*3 + 2*2
    ADDI GFX_PTR1, 4
    SUBI TMP, GFX_PTR1, GFX_MATRIXW+8*3
    BRFS .rotate_loop1

    ; Calculate Nodes
    LDIW RA, GFX_WIREFRAME_NODES
    LDI RB, GFX_WIREFRAME_NODES_LEN-1
    LDI GFX_PTR1, GFX_NODES
.compute_loop:
    LPM GFX_MATRIXA + 2*0, RA
    ADDI RA, 2
    ADCI RA+2, 0
    LPM GFX_MATRIXA + 2*1, RA
    ADDI RA, 2
    ADCI RA+2, 0
    LPM GFX_MATRIXA + 2*2, RA
    ADDI RA, 2
    ADCI RA+2, 0
    REV GFX_MATRIXA + 2*0
    REV GFX_MATRIXA + 2*1
    REV GFX_MATRIXA + 2*2

    ; Compute X, Y, Z
    LDI RC, 3-1
    LDI GFX_PTR2, GFX_MATRIXW
    LDI GFX_PTR3, GFX_MATRIXA + 8
.compute_matrix_loop1:
    LDI RD, 3-1
    MOV RE, GFX_PTR2
    LDI GFX_PTR4, GFX_MATRIXA
    LDIW RF, 0
.compute_matrix_loop2:
    LD FA, RE
    LD FB, GFX_PTR4
    CALL Multiply32_16x16_signed
    ADD RF, FA
    ADC RF+2, FA+2
    ADDI RE, 8
    ADDI GFX_PTR4, 2
    DEC RD
    BRFC .compute_matrix_loop2
    LDI FA, 0
    LD FA+2, RE     ; RE<<16
    ASR FA+2
    ROR FA
    ASR FA+2
    ROR FA          ; RE<<14
    ADD RF, FA
    ADC RF+2, FA+2
    ASR RF+2
    ASR RF+2
    ST GFX_PTR3, RF+2
    ADDI GFX_PTR2, 2
    ADDI GFX_PTR3, 2
    DEC RC
    BRFC .compute_matrix_loop1

    ; Get LCD location
    ADDI RC, GFX_MATRIXA + 8 + 2*2, GFX_FOV<<10
    MOV FA, GFX_MATRIXA + 8 + 2*0
    LDI FB, GFX_FOV*GFX_SCALE
    CALL Multiply32_16x16_signed
    MOV FB, RC
    CALL Divide32_32x16_signed
    ADDI FA, LCD_WIDTH/2
    STB GFX_PTR1, FA
    INC GFX_PTR1
    MOV FA, GFX_MATRIXA + 8 + 2*1
    LDI FB, GFX_FOV*GFX_SCALE
    CALL Multiply32_16x16_signed
    MOV FB, RC
    CALL Divide32_32x16_signed
    ADDI FA, LCD_HEIGHT/2
    STB GFX_PTR1, FA
    INC GFX_PTR1
    DEC RB
    BRFC .compute_loop

    ; DRAW lines
    LDIW RA, GFX_WIREFRAME_LINES
    LDI RB, GFX_WIREFRAME_LINES_LEN-1
.draw_loop:
    LPM RC, RA
    REV RC
    ANDI RD, RC+1, 0xFF ; first point
    LSL RD
    ADDI RD, GFX_NODES
    LD RD, RD
    ANDI FA, RD, 0xFF
    ANDI FB, RD+1, 0xFF
    ANDI RD, RC, 0xFF   ; second point
    LSL RD
    ADDI RD, GFX_NODES
    LD RD, RD
    ANDI FC, RD, 0xFF
    ANDI FD, RD+1, 0xFF
    CALL GFX_Line
    ADDI RA, 2
    ADCI RA+2, 0
    DEC RB
    BRFC .draw_loop

    CALL GFX_Update

    ; Rx registers are not restored
    POP_PC
    RET
