#once

#include "library.s"
#include "lcd5110.s"

; Library Functions
; DS1302_Read           | Read register FA16 = [FA16] 0~30
; DS1302_Write          | Write register [FA16] = FB8 0~30

; DS1302_Demo           | Run demo

;;;;;;;;;;;;;;;;;;;;;;;;
; Hardware connection:
; FLASH DATA -> DS1302 DATA (via 10k resistor)
; OP bit 6  -> DS1302 CLK
; OP bit 7  -> DS1302 CE
;;;;;;;;;;;;;;;;;;;;;;;;

; Special instruction sequences:
#ruledef
{
    DS1302_Write {addr: u8}, {val: u8} => asm {
        LDI FA, addr
        LDI FB, val
        CALL DS1302_Write
    }
    
    ShiftBy4 {dst: reg} => asm {
        LSR dst
        LSR dst
        LSR dst
        LSR dst
    }
}

; Read register FA16 = [FA16] 0~30
DS1302_Read:
    BRSHI FA, 31, .stop
    OUTI 0      ; NOP for PCD8544 LCD and RST for DS1302
    LSL FA
    ORI FA, 0x81
    LDI FB, 8-1
.addr_loop:
    ANDI TMP, FA, 1
    BRFC .addr_set
    OUT_BIT 0
    JMP .addr_continue
.addr_set:
    OUT_BIT 1
.addr_continue:
    LSR FA
    DEC FB
    BRFC .addr_loop
    LDI FA, 0
    LDI FB, 1
.data_loop:
    ANDI TMP, 0, 0x01
    BRFS .data_cleared
    OR FA, FB
.data_cleared:
    LSL FB
    ANDI FB, 0xFF
    BRFS .stop
    OUT2 FA+1
    JMP .data_loop
.stop:
    RET

; Write register [FA16] = FB8 0~30
DS1302_Write:
    BRSHI FA, 31, .stop
    OUTI 0      ; NOP for PCD8544 LCD and RST for DS1302
    LSL FA
    ORI FA, 0x80
    OR FA+1, FB
    LDI FB, 16-1
.loop:
    ANDI TMP, FA, 1
    BRFC .set
    OUT_BIT 0
    JMP .continue
.set:
    OUT_BIT 1
.continue:
    LSR FA
    DEC FB
    BRFC .loop
.stop:
    RET

DS1302_Demo:
    PUSH_PC
    PUSH RA

    ; Set clock
    ;CALL WaitForButton
    ;DS1302_Write 7, 0   ; Enable write
    ;DS1302_Write 8, 0   ; No trickle charge
    ;DS1302_Write 0, (0<<4) | 0 ; Seconds
    ;DS1302_Write 1, (3<<4) | 0 ; Minutes
    ;DS1302_Write 2, (1<<4) | 7 ; Hours
    ;DS1302_Write 3, (0<<4) | 4 ; Day
    ;DS1302_Write 4, (0<<4) | 5 ; Month
    ;DS1302_Write 5, 5   ; Day of week
    ;DS1302_Write 6, (2<<4) | 3 ; Year
    ;DS1302_Write 7, 0x80 ; Disable write

.loop:

    LCD_SetPos 3, 2
    LDI FA, 2   ; Hours
    CALL DS1302_Read
    ANDI RA, FA, 0xF
    ShiftBy4 FA
    ANDI FA, 0x3
    ADDI FA, 48
    CALL LCD_WriteChar
    ADDI FA, RA, 48
    CALL LCD_WriteChar
    LCD_WriteChar 58
    LDI FA, 1   ; Minutes
    CALL DS1302_Read
    ANDI RA, FA, 0xF
    ShiftBy4 FA
    ANDI FA, 0x7
    ADDI FA, 48
    CALL LCD_WriteChar
    ADDI FA, RA, 48
    CALL LCD_WriteChar
    LCD_WriteChar 58
    LDI FA, 0   ; Seconds
    CALL DS1302_Read
    ANDI RA, FA, 0xF
    ShiftBy4 FA
    ANDI FA, 0x7
    ADDI FA, 48
    CALL LCD_WriteChar
    ADDI FA, RA, 48
    CALL LCD_WriteChar

    LCD_SetPos 2, 3
    LDI FA, 3   ; Days
    CALL DS1302_Read
    ANDI RA, FA, 0xF
    ShiftBy4 FA
    ANDI FA, 0x3
    ADDI FA, 48
    CALL LCD_WriteChar
    ADDI FA, RA, 48
    CALL LCD_WriteChar
    LCD_WriteChar 47
    LDI FA, 4   ; Month
    CALL DS1302_Read
    ANDI RA, FA, 0xF
    ShiftBy4 FA
    ANDI FA, 0x1
    ADDI FA, 48
    CALL LCD_WriteChar
    ADDI FA, RA, 48
    CALL LCD_WriteChar
    LCD_WriteChar 47
    LCD_WriteChar 50
    LCD_WriteChar 48
    LDI FA, 6   ; Year
    CALL DS1302_Read
    ANDI RA, FA, 0xF
    ShiftBy4 FA
    ANDI FA, 0x7
    ADDI FA, 48
    CALL LCD_WriteChar
    ADDI FA, RA, 48
    CALL LCD_WriteChar

    JMP .loop

    POP RA
    POP_PC
    RET
