#once

#include "library.s"

; Library Functions
; LCD8833_WriteData     | Write 8-bit data in FA
; LCD8833_WriteCMD      | Write 8-bit command in FA

; LCD_Init              | Initialize the LCD
; PCF8833_Demo          | Run demo

;;;;;;;;;;;;;;;;;;;;;;;;
; Hardware connection:
; FLASH DATA -> PCF8833 DATA (via 10k resistor)
; OP bit 6  -> PCF8833 CLK
; OP bit 7  -> PCF8833 CS
;;;;;;;;;;;;;;;;;;;;;;;;

; Special instruction sequences:
#ruledef
{
    LCD8833_WriteCMD {cmd: u8} => asm {
        LDI FA, cmd
        CALL LCD8833_WriteCMD
    }
}

; Write data [FA16]
LCD8833_WriteData:
    OUT_BIT 1
LCD8833_Write:
    LDI FB, 8-1
.loop:
    ANDI TMP, FA, 0x80
    BRFC .set
    OUT_BIT 0
    JMP .continue
.set:
    OUT_BIT 1
.continue:
    LSL FA
    DEC FB
    BRFC .loop
.stop:
    RET

; Write command [FA16]
LCD8833_WriteCMD:
    OUT_BIT 0
    JMP LCD8833_Write

; Initialize LCD
LCD_Init:
    PUSH_PC
    OUTI 0      ; NOP for PCD8544 LCD and RST for PCD8833
    LCD8833_WriteCMD 0x01   ; Soft reset
    LCD8833_WriteCMD 0x11   ; Sleep out
    LCD8833_WriteCMD 0x03   ; Booster on
    DelayCycles delay8MHz_ms(10)
    LCD8833_WriteCMD 0x29   ; Display on
    LCD8833_WriteCMD 0x13   ; Normal display mode
    LCD8833_WriteCMD 0x20   ; Display inversion on
    LCD8833_WriteCMD 0xBA   ; Data order
    LCD8833_WriteCMD 0x36   ; Memory access control
    LCD8833_WriteCMD 0x08   ; No mirror
    LCD8833_WriteCMD 0x3A   ; 16-bit per pixel
    LCD8833_WriteCMD 0x05   ; 16-bit per pixel
    ;LCD8833_WriteCMD 0x25   ; Contrast
    ;LCD8833_WriteCMD   60   ; Contrast
    LCD8833_WriteCMD 0x2C   ; Memory write
    OUTI 0      ; NOP for PCD8544 LCD and RST for PCD8833
    POP_PC
    RET

LCD_Clear:
    RET

PCF8833_Lenna:
    #d incbin("lenna.bin")

PCF8833_Demo:
    PUSH_PC
    PUSHW RA
    PUSH RB

    ; Takes 96 seconds
    LDIW RA, PCF8833_Lenna+4
    LDI RB, 98*70-1
.loop:
    LPM FC, RA
    ADDI32 RA, 2
    REV FC
    MOV FA, FC+1
    CALL LCD8833_WriteData
    MOV FA, FC
    CALL LCD8833_WriteData
    DEC RB
    BRFC .loop

    HALT
    POP RB
    POPW RA
    POP_PC
    RET
