// By Jiri Stepanovsky - jiristepanovsky.cz
// Build: 21.03.2023
// License: MIT (see License.txt)

#include <avr/io.h>

.global asm_write_char
.global asm_read_char
.section .text

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Uart output, baud 19200 at 8 MHz
;     R25 - byte to send
;     R24 - bit to send
;     R23 - tmp reg
;
; extern void asm_write_char(uin8_t)
; extern uint8_t asm_read_char()
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Specify delay in exact delay loop cycles
; UART_DELAY = F_CPU/BAUD/3
; must be <= 255

#define UART_SET_1  sbi 0x18, 3
#define UART_SET_0  cbi 0x18, 3
#define UART_SBIC   sbic 0x16, 3
#define UART_DELAY  153

#define UART_DATA   r24
#define UART_BIT    r25
#define UART_TMP    r23

asm_write_char:
    UART_SET_0                      ; start bit
    ; Attiny10: (4 + 3*UART_DELAY) or (5 + 3*UART_DELAY) cycles
    ; Attiny13: (5 + 3*UART_DELAY) or (6 + 3*UART_DELAY) cycles
    ldi UART_TMP, lo8(UART_DELAY-1)
write_uart_startbit:
    dec UART_TMP
    brne write_uart_startbit
    ldi UART_BIT, 8
write_uart_mainloop:
    sbrc UART_DATA, 0
    rjmp write_uart_bit1
    UART_SET_0                      ; bit 0
    rjmp write_uart_shift
write_uart_bit1:
    UART_SET_1                      ; bit 1
    nop
write_uart_shift:                   ;
    lsr UART_DATA
    ; Attiny 10: 9 cycles
    ;ldi UART_TMP, lo8(UART_DELAY-3)
    ; Attiny 13: 12 cycles
    nop
    nop
    ldi UART_TMP, lo8(UART_DELAY-4)
write_uart_waitbit:
    dec UART_TMP
    brne write_uart_waitbit
    dec UART_BIT
    brne write_uart_mainloop        ; send 8 bits
    UART_SET_1
    ldi UART_TMP, lo8(UART_DELAY)   ; 1 stop bit
write_uart_stopbit:
    dec UART_TMP
    brne write_uart_stopbit
    ret

asm_read_char:
    ldi UART_BIT, 1
    ldi UART_DATA, 0
    ldi UART_TMP, lo8(UART_DELAY/2-1) ; Half-bit
read_uart_startbit:
    UART_SBIC                       ; Skip if zero (Start bit)
    rjmp read_uart_startbit
read_uart_waithalfbit:              ; Wait half-bit
    dec UART_TMP
    brne read_uart_waithalfbit
read_uart_mainloop:                 ; Main Loop
    ldi UART_TMP, lo8(UART_DELAY-3)
read_uart_waitbit:                  ; Wait bit
    dec UART_TMP
    brne read_uart_waitbit
    UART_SBIC                       ; Sample
    rjmp read_uart_bit1
read_uart_bit0:
    rjmp read_uart_shift
read_uart_bit1:
    or UART_DATA, UART_BIT
read_uart_shift:
    nop
    nop
    lsl UART_BIT
    brne read_uart_mainloop         ; 9 cycles
    ldi UART_TMP, lo8(UART_DELAY)   ; 1 stop bit
read_uart_stopbit:
    dec UART_TMP
    brne read_uart_stopbit
    ret

